/*
 *  ActionHandler in org.jpws.front
 *  file: ActionHandler.java
 * 
 *  Project Jpws-Front
 *  @author Wolfgang Keller
 *  Created 06.09.2004
 *  Version
 * 
 *  Copyright (c) 2005 by Wolfgang Keller, Munich, Germany
 * 
 This program is not freeware software but copyright protected to the author(s)
 stated above. However, you can use, redistribute and/or modify it under the terms 
 of the GNU General Public License as published by the Free Software Foundation, 
 version 2 of the License.

 This program is distributed in the hope that it will be useful, but WITHOUT
 ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.

 You should have received a copy of the GNU General Public License along with
 this program; if not, write to the Free Software Foundation, Inc., 59 Temple
 Place - Suite 330, Boston, MA 02111-1307, USA, or go to
 http://www.gnu.org/copyleft/gpl.html.
 */

package jslovo.ui;


import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;


import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;

import jslovo.util.*;

/**
 *  Static singleton class organizing actions, mainly depending on the command 
 *  menus, and action related assisting routines. This class manages the system's
 *  idle time counter. 
 * <p>
 */
public class ActionHandler
{
   private static ActionListener actionListener = new MainActionListener();
   private static DocumentListener docListener = new DocumentChangeListener();
   private static long lastUseTime;
   private static Action clearClipAction;
   
   /**
    *  Singleton class. 
    */
   private ActionHandler ()
   {
   }

   public static void init (  )
   {
      lastUseTime = System.currentTimeMillis();
      
      // define Action "menu.edit.clearclip"
      clearClipAction = new AbstractAction( 
            ResourceLoader.getCommand( "menu.edit.clearclip" ),
            ResourceLoader.getImageIcon( "clearclip" ) )
      {
         public void actionPerformed( ActionEvent arg )
         {
            actionListener.actionPerformed( arg );
         }
      };
      clearClipAction.putValue( Action.SHORT_DESCRIPTION, 
            ResourceLoader.getCommand( "toolbar.clearclip.tooltip" ) );
      clearClipAction.putValue( Action.ACTION_COMMAND_KEY, "menu.edit.clearclip" );
      
   }  // init
   
   /** Returns the "ClearClipboard" action which is defined in this class. */
   public static Action getClearClipAction ()
   {
      return clearClipAction;
   }
   
   /** Returns the <code>ActionListener</code> which is defined in this class. */
   public static ActionListener getMainActionListener ()
   {
      return  actionListener;
   }

   /** Returns the actual idle time of the application. */ 
   public static long getIdleTime ()
   {
      return System.currentTimeMillis() - lastUseTime;
   }

   /** Resets the application's idle counter to zero (current time). */
   public static void resetIdleTime ()
   {
      lastUseTime = System.currentTimeMillis();
   }
   
   /** Causes a new password file to be created and displayed in the GUI.
    *  (Includes user passphrase input.)
    */



/**
 *  An instance of this class listenes to <code>JTextComponent</code>s and 
 *  resets the system's idle-time counter whenever a modification occurs on
 *  a component's text.      
 */
private static class DocumentChangeListener implements DocumentListener
{

public void changedUpdate ( DocumentEvent e )
{}

public void insertUpdate ( DocumentEvent e )
{
   resetIdleTime();
}

public void removeUpdate ( DocumentEvent e )
{
   resetIdleTime();
}
} // inner class DocumentChangeListener
 
/**
 *  This class is the main listener to action commands. Those are majorly
 *  coming from the GUI's menubars and toolbars.
 *  
 */
private static class MainActionListener implements ActionListener
{

public void actionPerformed ( ActionEvent arg0 )
{

   String cmd, hstr;
   boolean isSelected;
   
   lastUseTime = System.currentTimeMillis();
   cmd = arg0.getActionCommand();

//   System.out.println("Debug: action:"+cmd);  
 
   if ( cmd.equals( "" ) )
      return;
  
 
   else if ( cmd.equals( "menu.help.about" ) )
   {
       new AboutDialog();
   }

  else if ( cmd.equals( "menu.help.docs" ) )
   {
       new HelpDialog();
   }

 else if ( cmd.equals( "menu.file.exit" ) )
   {
       Global.mainFrame.exitApplication();
   }

else if ( cmd.equals( "menu.file.save" ) )
   {
       Global.mainFrame.SaveFileHTML();
   }

else if ( cmd.equals( "menu.tools.makedb" ) )
   {
       new jslovo.jdatabase.DBmaker();
   }


else if ( cmd.equals( "menu.tools.keyboard" ) )
   {
       new MakeKeyboard(); 
}
  

else if ( cmd.equals( "menu.options.fonts" ) )
   {
       Global.mainFrame.FontSelect(); 
}


else if ( cmd.equals( "menu.options.foreground" ) )
   {
       Global.mainFrame.ForegroundSelect();
}

else if ( cmd.equals( "menu.options.background" ) )
   {
       Global.mainFrame.BackgroundSelect();
}


else if ( cmd.equals( ResourceLoader.getCommand("button.search")  ) )
   {
       Global.mainFrame.Search();
}


 
   else if ( cmd.equals(ResourceLoader.getCommand("menu.exactmatch" )) )
   {
     // do not show the list  
      Global.sCand.setInvisible();
 
      Global.description="";
      Global.Cand.clear();
      Global.CandIndex.clear();
      Global.CandDic.clear();
      Global.CandTopIndex.clear();
   }
   
   else if ( cmd.equals(ResourceLoader.getCommand("menu.substring") ) )
   {

      // show the list
      Global.sCand.setVisible();
      Global.SelectSubstring=true;
   }
   
   else
      System.out.println("** unrecognized command: " + cmd );
   
}  // actionPerformed


}






}
